@@ -34,9 +34,9 @@
  * in return the list of FileItem structs.
  *
  * The recent files list can be filtered by calling
- * gedit_open_document_selector_store_set_recent_filter()
+ * gedit_open_document_selector_store_set_filter()
  * and you can get the actual filter by calling
- * gedit_open_document_selector_store_get_recent_filter()
+ * gedit_open_document_selector_store_get_filter()
  * ( this is in addition to the text mime type filter)
  *
  * The recent files list is not capped by Gedit settings like
@@ -67,14 +67,16 @@ struct _GeditOpenDocumentSelectorStore
 {
 	GObject parent_instance;
 
+	GSource *recent_source;
+
 	GeditRecentConfiguration recent_config;
+	gchar *filter;
 	GList *recent_items;
 	gint recent_config_limit;
 	gboolean recent_items_need_update;
 };
 
 G_LOCK_DEFINE_STATIC (recent_files_filter_lock);
-G_LOCK_DEFINE_STATIC (store_recent_items_lock);
 
 G_DEFINE_TYPE (GeditOpenDocumentSelectorStore, gedit_open_document_selector_store, G_TYPE_OBJECT)
 
@@ -82,7 +84,7 @@ G_DEFINE_QUARK (gedit-open-document-selector-store-error-quark,
                 gedit_open_document_selector_store_error)
 
 static GList *
-get_current_docs_list (GeditOpenDocumentSelectorStore *selector_store,
+get_current_docs_list (GeditOpenDocumentSelectorStore *selector_store G_GNUC_UNUSED,
                        GeditOpenDocumentSelector      *selector)
 {
 	GeditWindow *window;
@@ -172,7 +174,7 @@ check_mime_type (GFileInfo *info)
 }
 
 static GList *
-get_children_from_dir (GeditOpenDocumentSelectorStore *selector_store,
+get_children_from_dir (GeditOpenDocumentSelectorStore *selector_store G_GNUC_UNUSED,
                        GFile                          *dir)
 {
 	GList *file_items_list = NULL;
@@ -268,7 +270,7 @@ get_active_doc_dir_list (GeditOpenDocumentSelectorStore *selector_store,
 }
 
 static GFile *
-get_file_browser_root (GeditOpenDocumentSelectorStore *selector_store,
+get_file_browser_root (GeditOpenDocumentSelectorStore *selector_store G_GNUC_UNUSED,
                        GeditOpenDocumentSelector      *selector)
 {
 	GeditWindow *window;
@@ -350,7 +352,7 @@ read_bookmarks_file (GFile *file)
 
 static GList *
 get_local_bookmarks_list (GeditOpenDocumentSelectorStore *selector_store,
-                          GeditOpenDocumentSelector      *selector)
+                          GeditOpenDocumentSelector      *selector G_GNUC_UNUSED)
 {
 	GList *bookmarks_uri_list = NULL;
 	GList *file_items_list = NULL;
@@ -410,7 +412,7 @@ path_is_home_dir (const gchar *path)
 
 static GList *
 get_desktop_dir_list (GeditOpenDocumentSelectorStore *selector_store,
-                      GeditOpenDocumentSelector      *selector)
+                      GeditOpenDocumentSelector      *selector G_GNUC_UNUSED)
 {
 	GList *file_items_list = NULL;
 	const gchar *desktop_dir_name;
@@ -439,7 +441,7 @@ get_desktop_dir_list (GeditOpenDocumentSelectorStore *selector_store,
 
 static GList *
 get_home_dir_list (GeditOpenDocumentSelectorStore *selector_store,
-                   GeditOpenDocumentSelector      *selector)
+                   GeditOpenDocumentSelector      *selector G_GNUC_UNUSED)
 {
 	GList *file_items_list = NULL;
 	const gchar *home_name;
@@ -508,7 +510,7 @@ convert_recent_item_list_to_fileitem_list (GList *uri_list)
 
 static GList *
 get_recent_files_list (GeditOpenDocumentSelectorStore *selector_store,
-                       GeditOpenDocumentSelector      *selector)
+                       GeditOpenDocumentSelector      *selector G_GNUC_UNUSED)
 {
 	GList *recent_items_list;
 	GList *file_items_list;
@@ -526,7 +528,7 @@ get_recent_files_list (GeditOpenDocumentSelectorStore *selector_store,
 static void
 update_list_cb (GeditOpenDocumentSelectorStore *selector_store,
                 GAsyncResult                   *res,
-                gpointer                        user_data)
+                gpointer                        user_data G_GNUC_UNUSED)
 {
 	GList *list;
 	GError *error;
@@ -541,15 +543,12 @@ update_list_cb (GeditOpenDocumentSelectorStore *selector_store,
 	switch (type)
 	{
 		case GEDIT_OPEN_DOCUMENT_SELECTOR_RECENT_FILES_LIST:
-			G_LOCK (store_recent_items_lock);
-
 			gedit_open_document_selector_free_file_items_list (selector_store->recent_items);
 			selector_store->recent_items = list;
 
-			DEBUG_SELECTOR (g_print ("\tStore(%p): update_list_cb: Thread:%p, type:%s, length:%i\n",
-			                         selector_store, g_thread_self (), list_type_string[type], g_list_length (list)););
+			DEBUG_SELECTOR (g_print ("\tStore(%p): update_list_cb: type:%s, length:%i\n",
+			                         selector_store, list_type_string[type], g_list_length (list)););
 
-			G_UNLOCK (store_recent_items_lock);
 			break;
 		default:
 			break;
@@ -557,7 +556,7 @@ update_list_cb (GeditOpenDocumentSelectorStore *selector_store,
 }
 
 static void
-on_recent_manager_changed (GtkRecentManager *manager,
+on_recent_manager_changed (GtkRecentManager *manager G_GNUC_UNUSED,
                            gpointer          user_data)
 {
 	GeditOpenDocumentSelectorStore *selector_store = GEDIT_OPEN_DOCUMENT_SELECTOR_STORE (user_data);
@@ -578,6 +577,9 @@ gedit_open_document_selector_store_dispose (GObject *object)
 
 	gedit_recent_configuration_destroy (&selector_store->recent_config);
 
+	g_clear_pointer (&selector_store->recent_source, g_source_destroy);
+	g_clear_pointer (&selector_store->filter, g_free);
+
 	if (selector_store->recent_items)
 	{
 		gedit_open_document_selector_free_file_items_list (selector_store->recent_items);
@@ -610,11 +612,64 @@ static GList * (*list_func [])(GeditOpenDocumentSelectorStore *selector_store,
 	get_current_docs_list
 };
 
+static gboolean
+update_recent_list (gpointer user_data)
+{
+	GeditOpenDocumentSelectorStore *selector_store;
+	GeditOpenDocumentSelector *selector;
+	PushMessage *message;
+	/* The type variable is only used when debug code activated */
+	G_GNUC_UNUSED ListType type;
+	GList *file_items_list;
+	GTask *task = G_TASK(user_data);
+
+	selector_store = g_task_get_source_object (task);
+	message = g_task_get_task_data (task);
+	selector = message->selector;
+	type = message->type;
+
+	DEBUG_SELECTOR_TIMER_DECL
+	DEBUG_SELECTOR_TIMER_NEW
+	DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: type:%s\n",
+	                         selector, list_type_string[type]););
+
+	/* Update the recent list only when it changes, copy otherwise but keep it the first time */
+	if (selector_store->recent_items != NULL && selector_store->recent_items_need_update == FALSE)
+	{
+		file_items_list = gedit_open_document_selector_copy_file_items_list (selector_store->recent_items);
+
+		DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: recent list copy\n", selector););
+	}
+	else
+	{
+		selector_store->recent_items_need_update = FALSE;
+		file_items_list = get_recent_files_list (selector_store, selector);
+
+		DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: recent list compute\n", selector););
+
+		if (selector_store->recent_items == NULL)
+		{
+			selector_store->recent_items = gedit_open_document_selector_copy_file_items_list (file_items_list);
+		}
+	}
+
+	g_task_return_pointer (task,
+	                       file_items_list,
+	                       (GDestroyNotify)gedit_open_document_selector_free_file_items_list);
+
+	DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: type:%s, time:%lf\n",
+	                         selector, list_type_string[type], DEBUG_SELECTOR_TIMER_GET););
+	DEBUG_SELECTOR_TIMER_DESTROY
+
+        selector_store->recent_source = NULL;
+	return G_SOURCE_REMOVE;
+}
+
 static void
 update_list_dispatcher (GTask        *task,
                         gpointer      source_object,
                         gpointer      task_data,
-                        GCancellable *cancellable)
+                        GCancellable *cancellable G_GNUC_UNUSED)
 {
 	GeditOpenDocumentSelectorStore *selector_store = source_object;
 	GeditOpenDocumentSelector *selector;
@@ -631,43 +686,6 @@ update_list_dispatcher (GTask        *task,
 	DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: Thread:%p, type:%s\n",
 	                         selector, g_thread_self (), list_type_string[type]););
 
-	/* Update the recent list only when it changes, copy otherwise but keep it the first time */
-	if (type == GEDIT_OPEN_DOCUMENT_SELECTOR_RECENT_FILES_LIST)
-	{
-		G_LOCK (store_recent_items_lock);
-
-		if (selector_store->recent_items != NULL && selector_store->recent_items_need_update == FALSE)
-		{
-			file_items_list = gedit_open_document_selector_copy_file_items_list (selector_store->recent_items);
-
-			DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: recent list copy\n", selector););
-		}
-		else
-		{
-			selector_store->recent_items_need_update = FALSE;
-			file_items_list = get_recent_files_list (selector_store, selector);
-
-			DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: recent list compute\n", selector););
-
-			if (selector_store->recent_items == NULL)
-			{
-				selector_store->recent_items = gedit_open_document_selector_copy_file_items_list (file_items_list);
-			}
-		}
-
-		G_UNLOCK (store_recent_items_lock);
-
-		g_task_return_pointer (task,
-		                       file_items_list,
-		                       (GDestroyNotify)gedit_open_document_selector_free_file_items_list);
-
-		DEBUG_SELECTOR (g_print ("\tStore(%p): store dispatcher: Thread:%p, type:%s, time:%lf\n",
-		                         selector, g_thread_self (), list_type_string[type], DEBUG_SELECTOR_TIMER_GET););
-		DEBUG_SELECTOR_TIMER_DESTROY
-
-		return;
-	}
-
 	if (type >= GEDIT_OPEN_DOCUMENT_SELECTOR_LIST_TYPE_NUM_OF_LISTS)
 	{
 		g_task_return_new_error (task,
@@ -723,7 +741,17 @@ gedit_open_document_selector_store_update_list_async (GeditOpenDocumentSelectorS
 	g_task_set_priority (task, G_PRIORITY_DEFAULT);
 	g_task_set_task_data (task, message, (GDestroyNotify)g_free);
 
-	g_task_run_in_thread (task, update_list_dispatcher);
+	if (type == GEDIT_OPEN_DOCUMENT_SELECTOR_RECENT_FILES_LIST &&
+	    selector_store->recent_source == NULL)
+	{
+		selector_store->recent_source = g_idle_source_new ();
+		g_task_attach_source (task, selector_store->recent_source, update_recent_list);
+	}
+	else
+	{
+		g_task_run_in_thread (task, update_list_dispatcher);
+	}
+
 	g_object_unref (task);
 }
 
@@ -757,8 +785,8 @@ gedit_open_document_selector_store_get_recent_limit (GeditOpenDocumentSelectorSt
 }
 
 void
-gedit_open_document_selector_store_set_recent_filter (GeditOpenDocumentSelectorStore *selector_store,
-                                                      const gchar                    *filter)
+gedit_open_document_selector_store_set_filter (GeditOpenDocumentSelectorStore *selector_store,
+                                               const gchar                    *filter)
 {
 	gchar *old_filter;
 
@@ -767,22 +795,22 @@ gedit_open_document_selector_store_set_recent_filter (GeditOpenDocumentSelectorS
 
 	G_LOCK (recent_files_filter_lock);
 
-	old_filter = selector_store->recent_config.substring_filter;
-	selector_store->recent_config.substring_filter = g_strdup (filter);
+	old_filter = selector_store->filter;
+	selector_store->filter = g_strdup (filter);
 
 	G_UNLOCK (recent_files_filter_lock);
 	g_free (old_filter);
 }
 
 gchar *
-gedit_open_document_selector_store_get_recent_filter (GeditOpenDocumentSelectorStore *selector_store)
+gedit_open_document_selector_store_get_filter (GeditOpenDocumentSelectorStore *selector_store)
 {
 	gchar *recent_filter;
 
 	g_return_val_if_fail (GEDIT_IS_OPEN_DOCUMENT_SELECTOR_STORE (selector_store), NULL);
 
 	G_LOCK (recent_files_filter_lock);
-	recent_filter = g_strdup (selector_store->recent_config.substring_filter);
+	recent_filter = g_strdup (selector_store->filter);
 	G_UNLOCK (recent_files_filter_lock);
 
 	return recent_filter;
